#!/usr/bin/tclsh
#
# Script: QIParser.tcl
# Requires: QIPGlobals.resource
#
#	 Copyright q.i. productions 2001
#
# This miniapp is distributed under a pretty flexible licence, namely:
# you must adopt my belief system, which more likely than not involves
# a greater, single (or more) 'Supreme Wombat' that oversees all Life and
# happenings with the aforementioned Life.
# Just kidding. If you should happen to redistribute this script in part
# or whole, we'd appreciate it if you kept this notice and acknowledged 
# our part in the whole grand scheme of things, even if it just be around
# to your colleagues.
# We're not legally responsible for any changes you may make to the code that
# make it either a menace to society or injure some other party in any fashion,
# really. 
# Now, you're not legally bound to any of this, really, you're just going
# to lose major karma points if you do anything stupid (in the nasty sense).
#
# This script should be a relatively simple parser of the relatively
# simple Sane Magazine horoscopes file format, as defined at 
# http://www.sanemagazine.com/publish/formats/horos-0.5.dtd
#
# Please see the README.rtf file for more information on this script
# and how to use it.
#
# A full version of the licence notice is available online at
#     http://supertart.com/software/licence.html
# Write us and we'll send you stickers.
#
# Begin. _m 01.01.01

set VERSION "1.0"

## Any script that uses any of these procs should source QIPGlobals.tcl
## at some point before sourcing this one...

  ######################################################################
  ## This proc isn't as sophisticated as Steve Ball's XML parsing package
  ## (http://www.zveno.com/zm.cgi/in-tclxml/); this is for a specific, limited
  ## set of tasks. It is a non-validating (largely) parser. For a more fully 
  ## featured XML parser, see the URI above, this script should suffice
  ## for most basic content distribution needs.
  ##
  ## The args (data) passed is the very long string obtained from
  ## QIP_H:loadSource.
  ##
  ## To determine the channel scope, string first (opening tag) string last
  ## (closing tag) are used to set the string range, whereas with every
  ## other element string first string first is used, because we want the next
  ## instance of that tag closing, instead of looking from the end of the
  ## string.
  ## These notes were largely for the RSS version of this script:
  ## (QIPRSSParse.tcl)
  ######################################################################

 proc horosParse {data {flags NULL}} {

 ## See end for the different arrays and elements you'll be left with
 ## at the end of all of this. The following sets the variables you'll
 ## have access to if you choose to write your own proc for HTML (or otherwise)
 ## formatting later.
 ## For true flexibility and to avoid killing the MVC paradigm, you should
 ## use the -t displayTemplate flag, where displayTemplate is the name of
 ## a file with the structure you'd like the content to be displayed in,
 ## variables to be substituted as is, inline, so mind your typing.

 global qiHContent qiHoroscopes qiHSign numSigns

 ## First off, check if this is a valid Horos file. Initially, we shouldn't
 ## have to worry about this.
 ## If this is -1, it's not a horoscopes XML file, or not against our criteria,
 ## which is terribly crude (or simple, our Thoreavian friends may insist),
 ## at the moment. 
 ## That said, in the fledgling Java port (back), a degree *more* validation
 ## is done.
 ## Otherwise, all things looking okay thus far, we'll now have access to a
 ## long string of rubbish.

 set validFile [string first "DOCTYPE horoscopes" $data]

 if {$validFile < 0} {
   puts "This doesn't seem to be a valid horoscopes file."
   puts "\n\nWe're getting out of here...\n\n"
   exit
 }

 ## Now we isolate the content from all the other junk in the file (comments,
 ## xml declarations, horoscopes DTD ref, &c.) (By junk, of course, I mean non-
 ## content-like stuff...)

 set qiHContent [string range $data [string first "<horoscopes>" $data] [expr [string last "</horoscopes>" $data] + 12]]
 set endOfContent [string last "</horoscopes>" $data]
#puts $qiHContent

 ## The following defines the horoscopes file format
 set qiHoroscopesPCDATA [list "title" "description" "link" "language" "sign"]
 set qiHSignAttrs [list "name"]
 set qiHSignPCDATA [list "prediction" "daterange" "link"]

 foreach element $qiHoroscopesPCDATA {
    if {[string compare $element "sign"] != 0} {
	set qiHoroscopes($element) [string range $qiHContent [string first "<$element>" $qiHContent] [string first "</$element>" $qiHContent]]
	set qiHoroscopes($element) [string trimright $qiHoroscopes($element) <]
 	regsub "<$element>" $qiHoroscopes($element) "" qiHoroscopes($element)
	#puts qiHoroscopes($element)
	#puts $qiHoroscopes($element)
    } else {
	## What follows is the messy stuff you have to do to dig through
	## the mess of an XML file you've created.
	set n 0
	set startRange [string first "<sign " $qiHContent]
	set endRange [string first "</sign>" $qiHContent]
	set qiHEntireSign [string trimright [string range $qiHContent [string first "<sign" $qiHContent] [string first "</sign>" $qiHContent]] <]
	## set a (r)evisedContent variable for this branch
	set rContent $qiHContent
	set i $endRange

	while {$i != -1} {
	  incr n
	  set signName [string range $rContent [expr [string first "name=\"" $rContent]+6] [string wordend $rContent [expr [string first "name=\"" $rContent]+7]]]
	  set signName [string trimright $signName \"]
	  set qiHSign($signName) [string range $rContent $startRange $endRange]
	  set qiHSign($signName,title) $signName
	  foreach item $qiHSignPCDATA {
	     set qiHSign($signName,$item) [string range $rContent [string first "<$item>" $rContent] [string first "</$item>" $rContent]]
	    set qiHSign($signName,$item) [string trimright $qiHSign($signName,$item) <]
            regsub "<$item>" $qiHSign($signName,$item) "" qiHSign($signName,$item)
	    #puts "qiHSign($signName,$item): $qiHSign($signName,$item)"
	  }
	  set startRange [expr $endRange + 1]
	  set rContent [string range $rContent $startRange $endOfContent]
	  set startRange [string first "<sign " $rContent]
	  set endRange [string first "</sign>" $rContent]
	  set i $endRange
	}
    }
	
 }
 set numSigns $n
}
 ## EOP(parse)

 proc toHTML {} {
	## this proc is probably sitting in the component,
	## You should not use the routine below, as it's more
	## an example than anything. You should endeavour to
	## use the template file whenever possible.

global qiHoroscopes qiHSign numSigns QIPGlobalHoroscopesOutputFile QIPGlobalTemplateDirectory QIPGlobalHTMLHoroscopesTemplate
# need to get file, and it's channel with you open it
# (probably in w mode (not w+)
# and then puts ?channelID? "content" to it.
	set templ [open ${QIPGlobalTemplateDirectory}${QIPGlobalHTMLHoroscopesTemplate} r]
	set template [read $templ]
	close $templ
	set output [eval $template]
	return $output
 }
