#!/usr/bin/tclsh
#
# Script: QIPUtils.tcl
#
#	 Copyright q.i. productions 2001
#
# This script contains the loadSource, the enigmatic fileFormat, and toFile
# procs, for those of you looking to nick any of the code. 
# Or to clean it up, either way.
#
# A full version of the licence notice is available online at
#     http://supertart.com/software/licence.html
# Write us and we'll send you stickers.
#

    ## This proc is largely the same for all of these scripts,
    ## the QI_* bundle, and is a core piece of the framework for
    ## the Cocoa/WOA version of this app.
    ## loadSource simply goes off and goes an HTTP GET on
    ## a URI, usually provided as an iVar (internal variable to
    ## the app, meaning we provide it for you). You can override this
    ## internal variable with command line flags, if you like.
    ## The result of this get should be passed on to the QIP_H::parse
    ## procedure.
    ## You'll note that the preceding comments are innaccurate,
    ## and I won't tell you exactly where. This file mostly
    ## delegates parsing tasks to one of the 'plug-in' scripts.
    ## See the ObjC app for bundles and plug-ins.

 proc loadSource {content {uri NULL}} {
  # if a URI is given, open a socket connection to the
  # site and get the XML file, otherwise, it'll just assume
  # it can go ahead with the content as the XML content,
  # until it finds otherwise (like failing the parse test).
  if {[string compare $uri "NULL"] != 0} {
	if {[string first ":" $uri] != -1} {
		set uriBits [split : $uri]
		set host [lindex $uriBits 0]
		set port [lindex $uriBits 1]
	} else {
		set host $uri
		set port 80
	}
  	set sock [socket $host $port]
  	puts $sock "GET $content HTTP/1.0"
	puts $sock "Host: $host"
	puts $sock ""
	flush $sock
	set content [read $sock]
	close $sock
	return $content
  } else {
	set content $content

  }
  return $content
 }

    ## This proc is for future use only, and is not currently
    ## implemented as it defines the RSS
    ## file format, and not the Horoscopes XML format.
    ## It's left in so we look forward-thinking. I wouldn't
    ## actually trust this to do anything, if you ran it.

 proc fileFormat {fileType} {
   upvar $fileType localVar
   upvar $fileType.channel localVar.channel
   upvar $fileType.channel.image localVar.channel.image
   upvar $fileType.channel.sign localVar.channel.sign
   upvar $fileType.channel.textinput localVar.channel.textinput

   switch $fileType {
     default {
	set localVar [list channel]
	set localVar.channel [list title description link image url sign textinput name rating language width height copyright pubDate lastBuildDate docs managingEditor webMaster hour day skipHours skipDays]
	set localVar.channel.image [list title url link width height description]
	set localVar.channel.sign [list title url description]
	set localVar.channel.textinput [list title description name link]
	set localVar.channel.skipHours [list hour]
	set localVar.channel.skipDays [list day]
     }
   }
 }


    ## The old API toFile{dataToWrite} is not supported any longer, and
    ## calls to toFile should use the infinitely more sensible
    ## toFile{dataToWrite file} syntax. This is to replace needing to include
    ## the QIPGlobals.tcl file in this script, which got us the following:
    ## global QIPGlobalOutputFile QIPGlobalOutputDirectory

 proc toFile {dataToWrite file} {
    set channelID [open $file w]
    puts $channelID $dataToWrite
    close $channelID
 }


## In my somewhat grand(ish) tradition of useless comments,
## Now Listening to: The Velvet Underground, Wild Child.
## Time: a very sensible 20.30, on a bank holiday Monday
